package cn.iocoder.yudao.module.depository.dal.mysql.warehouse;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.module.depository.vo.warehouse.WarehouseRegionVO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Mapper;
import cn.iocoder.yudao.module.depository.vo.warehouse.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

/**
 * 仓库 Mapper
 *
 * @author wlh
 */
@Mapper
public interface WarehouseMapper extends BaseMapperX<WarehouseDO> {

    default PageResult<WarehouseDO> selectPage(WarehousePageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<WarehouseDO>()
                .eqIfPresent(WarehouseDO::getStatus, reqVO.getStatus())
                .eq(reqVO.getNodeId() != null, WarehouseDO::getNodeId, reqVO.getNodeId())
                .and(StringUtils.isNotEmpty(reqVO.getKeywords()),
                        i -> i.like(WarehouseDO::getNumber, reqVO.getKeywords())
                                .or()
                                .like(WarehouseDO::getTitleZh, reqVO.getKeywords())
                                .or()
                                .like(WarehouseDO::getTitleEn, reqVO.getKeywords())
                )
                .orderByDesc(WarehouseDO::getId));
    }

    default List<WarehouseDO> selectList(WarehouseExportReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<WarehouseDO>()
                .eqIfPresent(WarehouseDO::getNodeId, reqVO.getNodeId())
                .eqIfPresent(WarehouseDO::getPid, reqVO.getPid())
                .eqIfPresent(WarehouseDO::getType, reqVO.getType())
                .eqIfPresent(WarehouseDO::getNumber, reqVO.getNumber())
                .eqIfPresent(WarehouseDO::getTitleZh, reqVO.getTitleZh())
                .eqIfPresent(WarehouseDO::getTitleEn, reqVO.getTitleEn())
                .eqIfPresent(WarehouseDO::getVolume, reqVO.getVolume())
                .eqIfPresent(WarehouseDO::getAddressZh, reqVO.getAddressZh())
                .eqIfPresent(WarehouseDO::getAddressEn, reqVO.getAddressEn())
                .eqIfPresent(WarehouseDO::getZhou, reqVO.getZhou())
                .eqIfPresent(WarehouseDO::getGuojia, reqVO.getGuojia())
                .eqIfPresent(WarehouseDO::getSheng, reqVO.getSheng())
                .eqIfPresent(WarehouseDO::getShi, reqVO.getShi())
                .eqIfPresent(WarehouseDO::getHead, reqVO.getHead())
                .eqIfPresent(WarehouseDO::getTell, reqVO.getTell())
                .eqIfPresent(WarehouseDO::getTradeType, reqVO.getTradeType())
                .eqIfPresent(WarehouseDO::getFreight, reqVO.getFreight())
                .eqIfPresent(WarehouseDO::getRemark, reqVO.getRemark())
                .eqIfPresent(WarehouseDO::getSort, reqVO.getSort())
                .eqIfPresent(WarehouseDO::getStatus, reqVO.getStatus())
                .betweenIfPresent(WarehouseDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc(WarehouseDO::getId));
    }

    @ResultType(WarehouseRegionVO.class)
    @Select({
            "SELECT w.*, ",
            "r1.short_name as guojiaShort,",
            "r2.short_name as shiShort ",
            "FROM ecw_warehouse w ",
            "LEFT JOIN ecw_region r1 ",
            "ON w.guojia = r1.id ",
            "LEFT JOIN ecw_region r2 ",
            "ON w.shi = r2.id ",
            "WHERE w.id = #{id}"
    })
    WarehouseRegionVO findWarehouseRegion(@Param("id") Long id);

    @ResultType(List.class)
    @Select({
            "<script>",
            "SELECT if(#{lang} = 0, w.title_zh, w.title_en) ",
            "FROM ecw_warehouse w ",
            "WHERE ",
            "w.id in ",
            "<foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>#{id}</foreach>",
            "</script>"
    })
    List<String> findWarehouseNameByIds(@Param("ids") List<String> ids, @Param("lang") Integer lang);

    @ResultType(WarehouseDO.class)
    @Select({
            "<script>",
            "SELECT * ",
            "FROM ecw_warehouse w ",
            "LEFT JOIN ecw_region r ",
            "ON r.id = w.shi ",
            "WHERE ",
            " w.shi = #{cityId} ",
            "and w.status = 0 ",
            "and r.type in ",
            "<foreach item='type' index='index' collection='types' open='(' separator=',' close=')'>#{type}</foreach>",
            "</script>"
    })
    List<WarehouseDO> findWarehouseListByCityIdAndTypes(@Param("cityId") Long cityId, @Param("types") List<Integer> types);

    @ResultType(WarehouseDO.class)
    @Select({
            "<script>",
            "SELECT r.* ",
            "FROM ecw_region r",
            "LEFT JOIN ecw_warehouse w  ",
            "ON r.id = w.guojia ",
            "WHERE ",
            " w.id = #{warehouseId} ",
            "and w.status = 0 ",
            "</script>"
    })
    RegionDO getRegion(Long warehouseId);

    @ResultType(List.class)
    @Select({
            "SELECT w.*, ",
            "r1.title_zh as guojiaName,",
            "r2.title_zh as shiName ",
            "FROM ecw_warehouse w ",
            "LEFT JOIN ecw_region r1 ",
            "ON w.guojia = r1.id ",
            "LEFT JOIN ecw_region r2 ",
            "ON w.shi = r2.id ",
            "WHERE  w.deleted = 0 AND w.`status` =0 AND trade_type =#{tradeType}"
    })
    List<WarehouseTreeRegionVO> getWarehouseTreeRegionList(@Param("tradeType") Integer tradeType);

    @ResultType(List.class)
    @Select({
            "<script>",
            "SELECT w.*, ",
            "r1.title_zh as guojiaName,",
            "r2.title_zh as shiName ",
            "FROM ecw_warehouse w ",
            "LEFT JOIN ecw_region r1 ",
            "ON w.guojia = r1.id ",
            "LEFT JOIN ecw_region r2 ",
            "ON w.shi = r2.id ",
            "WHERE  w.deleted = 0 AND w.`status` =0 AND trade_type =1  ",
            "<when test = 'type == 1'>",
            "GROUP BY r1.title_zh",
            "</when>",
            "<when test = 'type == 2'>",
            "AND w.guojia in (${regionId})",
            "GROUP BY w.shi",
            "</when>",
            "<when test = 'type == 3'>",
            "<if test= 'regionId != null and regionId != \"\" '>",
            "AND w.shi in (${regionId})",
            "</if>",
            "GROUP BY w.title_zh",
            "</when>",
            "<when test = 'type == 4'>",
            "GROUP BY w.shi",
            "</when>",
            "<when test = 'type == 5'>",
            "GROUP BY w.title_zh",
            "</when>",
            "</script>"
            })
    List<WarehouseTreeRegionVO> getRegionList(@Param("type") Integer type,@Param("regionId") String regionId,
                                              @Param("destCountryId") String destCountryId,@Param("objectiveId") String objectiveId,
                                              @Param("destWarehouseId") String  destWarehouseId);
}
